#include "mainwin.h"

#include <stdlib.h>

#include <qlined.h>
#include <qmlined.h>
#include <qpushbt.h>
#include <qapp.h>
#include <qpopmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qregexp.h>
#include <qstring.h>
#include <qlistbox.h>
#include <qstrlist.h>
#include <qlayout.h>
#include <qchkbox.h> 
#include <qfile.h>

#include <strings.h>


MainWin::MainWin( char **argv, QWidget *parent, const char *name ) 
  : QWidget( parent, name )
{
  QString s( getenv( "HOME" ) );
  s += "/.deluxe";
  
  QFile 
    opt( s );
  
  if ( opt.open( IO_ReadOnly ) )
    {
      opt.readBlock( (char *) &inf, sizeof( info ) );
      opt.close ();
    }
  else
    bzero( &inf, sizeof( info ) );
        
  QPushButton 
    *quit = new QPushButton( "Quit", this, "quit" ),
    *games = new QPushButton( "Games", this, "games" ),
    *player = new QPushButton( "Player", this, "player" ),
    *idlebt = new QPushButton( "Idle", this, "idle" );

  QMenuBar
    *m = new QMenuBar(this, "menu");
  
  QPopupMenu 
    *file = new QPopupMenu(),
    *options = new QPopupMenu();
  
  
  xBoard = new Child( "xboard", argv );

  CHECK_PTR( file );
  m->insertItem( "File", file );
  file->insertItem( "Quit", qApp, SLOT(quit()), ALT+Key_Q );
  CHECK_PTR( options );
  m->insertItem( "Options", options );
  options->insertItem( "Options ...", this, SLOT( options() ), ALT+Key_O );
  
  opt_win = 0;
  

  resize( 530, 500 );
  setMinimumSize( 530, 500 );
  setMaximumSize( 530, 500 );
  quit->setGeometry( 385, 460, 100, 30 );
  quit->setFont( QFont( "Times", 18, QFont::Bold ) );
  
  games->setGeometry( 135, 460, 100, 30 );
  games->setFont( QFont( "Times", 18) );
  player->setGeometry( 10, 460, 100, 30 );
  player->setFont( QFont( "Times", 18) );
  idlebt->setGeometry( 260, 460, 100, 30 );
  idlebt->setFont( QFont( "Times", 18 ) );
    
  connect( quit, SIGNAL( clicked() ), qApp, SLOT( quit() ));
  connect( player, SIGNAL( clicked() ), this, SLOT( playerList() ));
  connect( games, SIGNAL( clicked() ), this, SLOT( gameList() ) );
  connect( idlebt, SIGNAL( clicked() ), this, SLOT( idleWin() ) );
  

  p_wait_update = 0;
  g_wait_update = 0;
  play_win=0;
  game_win=0;
    
  shouts = new QMultiLineEdit( this, "shouts" );
  shouts->setGeometry( 0, 30, 530, 400 );
  shouts->setReadOnly( TRUE );
        
  cmd_in = new CmdLine( this, "cmd_in" );
  cmd_in->setGeometry( 2, 435, 528, 20 );
  cmd_in->setFocus();

  connect( cmd_in, SIGNAL( returnPressed() ), this, SLOT( sendCommand() ) );

  g_sel=p_sel=0;
  idle = 0;
  logsal = 1;
    
  startTimer( 10 );
} 

void MainWin::closeOpt()
{
  if( opt_win )
    {
      opt_win->close ();
      delete opt_win;
      opt_win = 0;
    }
}

void MainWin::setOpt()
{
  inf.a_salut = auto_salut->isChecked();
  inf.a_beep = beep_on_salut->isChecked();
  inf.l_salut = login_salut->isChecked();
  inf.i_beep = beep_on_idle_tell->isChecked();
  closeOpt();
}


void MainWin::options()
{
  opt_win = new QWidget( 0,  "options" );
  opt_win->setCaption( "Options" );
  opt_win->resize( 180, 190 );
  
  QBoxLayout
    *layout = new QVBoxLayout( opt_win, 5, 20 ),
    *buttons = new QHBoxLayout( 2 );
    
  auto_salut = new QCheckBox( "auto salut", opt_win, "auto_salut" );
  login_salut = new QCheckBox( "login salut", opt_win, "login_salut" );
  beep_on_salut = new QCheckBox( "beep on salut", opt_win, "beep_on_salut" );
  beep_on_idle_tell = new QCheckBox( "beep on tell", opt_win, "beep_on_idle_tell" );
  
  auto_salut->setChecked( inf.a_salut );
  login_salut->setChecked( inf.l_salut );
  beep_on_salut->setChecked( inf.a_beep );
  beep_on_idle_tell->setChecked( inf.i_beep );
  
  QPushButton
    *set = new QPushButton( " Set  ", opt_win, "set" ),
    *cancel = new QPushButton( "Cancel",  opt_win, "cancel" );
  
  
  layout->addWidget( auto_salut );
  layout->addWidget( login_salut );
  layout->addWidget( beep_on_salut );
  layout->addWidget( beep_on_idle_tell );
  layout->addLayout( buttons );

  buttons->addWidget( set );
  buttons->addWidget( cancel );

  connect( set, SIGNAL( clicked() ), this, SLOT( setOpt() ));
  connect( cancel, SIGNAL( clicked() ), this, SLOT( closeOpt() ) );
  
    
  
  opt_win->show();
  
}



CmdLine::CmdLine( QWidget * parent, const char * name )
  :QLineEdit( parent, name )
{
  history = new QStrList();
  hpos = 0;
}

CmdLine::~CmdLine()
{
  delete history;
}

void CmdLine::keyPressEvent( QKeyEvent * e )
{
  QString str;
  
  switch (e->key())
    {
    case Key_Up:
      if (hpos > 0)
	{
	  str = history->at(--hpos);
	  if( !str.isNull() )
	    setText( str );
	}
      break;
    case Key_Down:
      if (hpos + 1< history->count())
	{
	  str = history->at(++hpos);
	  if( !str.isNull() )
	    setText( str );
	}
      else
	if (hpos == history->count())
	  setText( "" );
      break;
    default:
      e->ignore();
      QLineEdit::keyPressEvent( e );
    }
}

void CmdLine::addHis( QString s )
{
  history->append( s );
  hpos = history->count();
}


MainWin::~MainWin()
{
  QString s( getenv( "HOME" ) );
  s += "/.deluxe";
  
  QFile opt( s );
  if( opt.open( IO_WriteOnly ) )
    {
      opt.writeBlock( (char *) &inf, sizeof( info ) );
      opt.close();
    }
  
  delete xBoard;
}

void MainWin::idleWin()
{
  if( !idle )
    {
      idleWidget = new QWidget( 0, "Idle" );

      QPushButton 
	*ok = new QPushButton( "Ok", idleWidget, "ok" );

      idleWidget->resize( 300, 55 );
      
      idleEd= new QLineEdit( idleWidget, "Idle" );
      idleEd->setCaption( "IdleText" );
      idleEd->resize( 300, 20 );
      
      ok->setGeometry( 125, 25, 50, 25 );

      connect( ok, SIGNAL( clicked () ), this, SLOT( closeIdle () ) );
      idle = 1;
      idleWidget->show();
      
    }
}


void MainWin::closeIdle()
{
  idleWidget->close();
  delete idleWidget;
  idle = 0;
}


void MainWin::parse( QString str )
{
  int start, stop,  emp=0;
  QString name;
  
  
  if( (start = stop = str.find( "tells" ) ) > 0)
    {
      do
	if( str[--start] == ' ' )
	  emp++;     
	else
	  if( str[start] == '(' )
	    stop = start;
      while( start > 0 && str[start] != '\n' );

      if ( str[start] == '\n' )
	start++;
      if ( emp < 2 )
	{
	  name=str.mid(start, stop); 
	  if( idle )
	    {
	      xBoard->put( "tell " );
	      xBoard->put( name );
	      xBoard->put( ' ' );
	      xBoard->put( idleEd->text () );
	      xBoard->put( '\n' );
	      if ( inf.i_beep )
		qApp->beep();
	    }
	  
	}
    }
  if ( inf.a_salut )
    if( str.find( "Notification:" ) == 0 && str.find( "arrived") != -1 )
      if( str.find( "adjourn" ) == -1 )
	{
	  int i=14;
	  xBoard->put( "shout " );
	  while ( str[i] != ' ' )
	    xBoard->put( str [i++] );
	  xBoard->put( "!\n");
	  if( inf.a_beep )
	    qApp->beep();
	}

  if ( inf.l_salut )
    if( logsal && (start = str.find("Present")) != -1 )
      if ( start == 0 || str[start-1] == '\n' )
	{
	  int i=start+25;
	  xBoard->put( "shout " );
	  while( str[++i] != '.' )
	    if ( (str [i] != ' ' || str[i-1] != ' ') &&
		str [i] != '\\' && str [i] != '\n' )
	      xBoard->put( str[i] );
	  xBoard->put( "!\n" );
	  logsal = 0; // znotify 
	}
}

	  
      

void MainWin::timerEvent( QTimerEvent * )
{
  char s[50000];
  int i=0,p=0, rpos=0, spos=0, g_pos=-1, g_stop=0, g_len=0;
  
  
  while ( i < 49999 && 
	  (xBoard->ready() ||
	   (p_wait_update && p!=0 && p!=3) ||
	   (g_wait_update && g_pos >= 0 && !g_stop ) ) )
    {
      xBoard->get( s [i] );
      if (i)
	{
	  if (p_wait_update)
	    if (s [i - 1] == '+')
	      if (s[i] == '\n')
		if (++p == 2) 
		  rpos = i+1;
		else;
	      else
		if (p == 1)
		  spos = i;
	  if (g_wait_update && !g_stop)
	    if (i >= 2)
	      if (i == 2 || s[i - 3] == '\n')
		if (s[i] <= '9' && s[i] >= '0')
		  if (g_pos < 0)
		    g_pos = i-2;
		  else
		    if (!g_len)
		      g_len = i - (g_pos + 2);
		    else;
		  else
		    g_stop = 1;
	}
      
      
      if (s[i] != '\n' || i > 0 )
	i++;
    }
  
  if (i)
    {
      unsigned int j;
      
      s[i] = 0;
      QString str( s );
      QRegExp num ("[0-9]");

      parse( str );
      
      if (str.find ("S_BEEP") != -1)
	qApp->beep();

      if (g_wait_update && str.find( "There are no games in progress.") != -1)
	g_wait_update = 0;
      
      if (rpos)
	{
	  int len = rpos - spos + 2;
	  Fics = len < 80 ? 0 : 2;
	  
	  j=rpos;
	  
	  play->setAutoUpdate( FALSE );
	  while (j < str.length() && s[j+10] != ' ')
	    {
	      QString 
		sub = str.mid( j+2, len - 4 );
	      
	      play->inSort( sub );
	      j+=len;
	    }
	  play->setAutoUpdate( TRUE );
	  play->repaint();
	  p_wait_update = 0;
	}
      
      if (g_pos>=0)
	{
	  unsigned int j = g_pos;
	  unsigned int str_len = str.length();
	  int again=1;
	  
	  game->setAutoUpdate( FALSE );
	  
	  while( j < str_len && again )
	    {
	      char sub[200];
	      int i=0;
	      
	      while( str[j] != '\n' && j < str_len )
		sub[i++] = str[j++];
	      j++;
	      sub[i] = 0;
	      
	      if ((again = sub[0] != 0))
		game->inSort( sub );
	    }
	  game->setAutoUpdate( TRUE );
	  game->repaint();
	  g_wait_update = 0;
	}
      
      
      if ( !rpos && g_pos < 0 )
	{
	  if (!str.isNull())
	    shouts->append( str );
	  //	  if (s [str.length()] == '\n')
	  shouts->setCursorPosition( 
				    shouts->numLines()-1, 
				    0, FALSE 
				    );
	}
    }
}


void MainWin::sendCommand()
{
  QString str( cmd_in->text() );
  xBoard->put( str );
  xBoard->put( '\n' );
  cmd_in->addHis( str );
  cmd_in->setText( "" );
}
 
void MainWin::playerList()
{
  if (play_win)
    {
      play_update();
      return;
    }

  play_win = new QWidget (0, "player");
  QPushButton 
    *update = new QPushButton( "Update", play_win, "update" ), 
    *match = new QPushButton( "Match", play_win, "match" ),
    *finger = new QPushButton( "Finger", play_win, "finger" ),
    *obs = new QPushButton( "Observe", play_win, "obs" ),
    *close = new QPushButton( "Close", play_win, "close" );
  
  play_win->resize(400, 310);
  update->setGeometry( 10, 275, 70, 30 );
  obs->setGeometry( 85, 275, 70, 30 );
  match->setGeometry( 160, 275, 70, 30 );
  finger->setGeometry( 240, 275, 70, 30 );
  close->setGeometry( 315, 275, 70, 30 );

  play = new QListBox( play_win, "player" );
  play->setGeometry( 0, 0, 400, 270 );
  
  play_update ();

  connect( close, SIGNAL( clicked() ), this, SLOT( play_close() ) );
  connect( update, SIGNAL( clicked() ), this, SLOT( play_update() ) );
  connect( match, SIGNAL( clicked() ), this, SLOT( make_match() ) );
  connect( play, SIGNAL( highlighted( int ) ), this, SLOT( p_select(int) ) );
  connect( finger, SIGNAL( clicked() ), this, SLOT( xfinger() ) );
  connect( obs, SIGNAL( clicked() ), this, SLOT( p_observe() ) );

  play_win->setCaption( "Player" );
  play_win->show(); 

}

void MainWin::gameList()
{
  if (game_win)
    {
      game_update();
      return;
    }

  game_win = new QWidget();
  QPushButton 
    *close = new QPushButton( "Close", game_win, "close" ),
    *update = new QPushButton( "Update", game_win, "update" ),
    *obs = new QPushButton( "Observe", game_win, "observe" );
  
  game_win->resize( 400, 300 );
  game_win->setCaption( "Games" );
  
  game = new QListBox( game_win, "games" );
  game->setGeometry( 0, 0, 400, 260 );
    
  close->setGeometry( 320, 265, 75, 30 );
  update->setGeometry( 10, 265, 75, 30 );
  obs->setGeometry( 165, 265, 75, 30 );
  
  game_update();

  connect( close, SIGNAL( clicked() ), this, SLOT( game_close() ) );
  connect( obs, SIGNAL( clicked() ), this, SLOT( g_observe() ) );
  connect( update, SIGNAL( clicked() ), this, SLOT( game_update() ) );
  connect( game, SIGNAL( highlighted(int) ), this, SLOT( g_select(int) ) );
  

  game_win->show();
}

  

void MainWin::play_update()
{
  xBoard->put ("who v\n");
  p_wait_update = 1;
  play->clear();
}

void MainWin::game_update()
{
  xBoard->put( "games\n" );
  g_wait_update = 1;
  game->clear();
}

  
void MainWin::play_close()
{
  play_win->close();
  delete play_win;
  play_win = 0;
}

void MainWin::game_close()
{
  game_win->close();
  delete game_win;
  game_win=0;
}


void MainWin::p_select( int i )
{
  p_sel = QString( play->text( i ) ).mid( 6+Fics, 6 + 17 );
}

void MainWin::g_select( int i )
{
  g_sel = QString( game->text( i ) ).left( 3 );
}


void MainWin::make_match()
{
  int i=0;
  if (p_sel)
    {
      
      xBoard->put( "match ");
      while( p_sel[i] != '('  && i < 17)
	xBoard->put( p_sel[i++] );
      xBoard->put( '\n' );
    }
  
}

void MainWin::p_observe()
{
  int i = 0;
  if (p_sel)
    {
      xBoard->put( "obs " );
      while( p_sel[i] != '(' && i < 17 )
	xBoard->put( p_sel[i++] );
      xBoard->put( '\n' );
    }
}

void MainWin::g_observe()
{
  if (g_sel)
    {
      xBoard->put( "obs " );
      xBoard->put( g_sel );
      xBoard->put( '\n' );
    }
}

  

void MainWin::xfinger()
{
  int i=0;
  
  if (p_sel)
    {
      xBoard->put( "finger " );
      while( p_sel[i] != '(' && i < 17 )
	xBoard->put( p_sel[i++] );
      xBoard->put( '\n' );
    }
}








